/* libMikMod ported to psl1ght by NoNameNo
   it can play .mod .xm .it .s3m file.
 */
#include <audio/audio.h>
#include <io/pad.h>
#include <psl1ght/lv2/timer.h>
#include <math.h>
#include <assert.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "mikmodsrc/include/mikmod_build.h"
#include "elysium.bin.h"

#define PI 3.14159265f

typedef struct _MOD_READER {
    MREADER Core;       /**< Structure with a bunch of pointers to functions. */
    unsigned long long     Offset;     /**< Current file position. */
    char    *BufferPtr; /**< Pointer to the music data. */
    unsigned long long     Size;       /**< Size of the music data. */
} MOD_READER;
static MODULE *module = NULL;   /**< Module structure. */


u32 playOneBlock(u64 *readIndex, float *audioDataStart)
{
	unsigned short samples[256 * 2];
	static u32 audio_block_index=1;

	u64 current_block = *readIndex;

	if(audio_block_index == current_block)
	{
		return 0;
	}

	float *buf = audioDataStart + 2 /*channelcount*/ * AUDIO_BLOCK_SAMPLES * audio_block_index;
	setBuffer((short *)samples, 256);
        MikMod_Update();

	for (int i = 0; i < 256 * 2; i++){
		buf[i] = (float)samples[i]/32767.0 ;
	}

	audio_block_index = (audio_block_index + 1) % AUDIO_BLOCK_8;
	return 1;
}

/**
 * This function has the same behaviour as feof.
 */
static bool MikMod_Eof(MREADER * reader)
{
    MOD_READER *pReader = (MOD_READER *) reader;

    return (pReader->Size == (pReader->Offset)) ? true : false;
}

/**
 * This function copies length bytes of data into dest, and return zero if an error occured, and any nonzero value otherwise. Note that an end-of-file condition will not be considered as an error$
 */
static bool MikMod_Read(MREADER * reader, void *ptr, size_t size)
{
    MOD_READER *pReader = (MOD_READER *) reader;

    memcpy(ptr, pReader->BufferPtr + pReader->Offset, size);
    pReader->Offset += size;

    return 1;
}

/**
 * This function has the same behaviour as fgetc.
 */
static int MikMod_Get(MREADER * reader)
{
    MOD_READER *pReader = (MOD_READER *) reader;
    char buf;

    buf = *(pReader->BufferPtr + pReader->Offset);
    pReader->Offset++;

    return((int)buf);
}

/**
 * This function has the same behaviour as fseek, with offset 0 meaning the start of the object (module, sample) being loaded.
 */
static bool MikMod_Seek(MREADER * reader, long offset, int whence)
{
    MOD_READER *pReader = (MOD_READER *) reader;

    if(whence == SEEK_SET)
        pReader->Offset = offset;
    else
        pReader->Offset += offset;

    return 1;
}

/**
 * This function has the same behaviour as ftell, with offset 0 meaning the start of the object being loaded.
 */
static long MikMod_Tell(MREADER * reader)
{
    MOD_READER *pReader = (MOD_READER *) reader;

    return pReader->Offset;
}


int main(int argc, const char* argv[])
{
	PadInfo padinfo;
        PadData paddata;

	MikMod_RegisterAllDrivers();
	MikMod_RegisterAllLoaders();

	md_mode = 0;
	md_mode |= DMODE_16BITS;
	md_mode |= DMODE_SOFT_MUSIC;
	md_mode |= DMODE_SOFT_SNDFX;
	md_mode |= DMODE_STEREO;
	md_mode |= DMODE_HQMIXER;

	md_mixfreq = 24000;

	MikMod_Init("");

	MOD_READER *Reader = (MOD_READER *)malloc(sizeof (MOD_READER));
    	if(Reader) {
        	Reader->Offset = 0;
	        Reader->BufferPtr = (char *)elysium_bin;
	        Reader->Size = sizeof(elysium_bin);
	        Reader->Core.Eof = &MikMod_Eof;
	        Reader->Core.Read = &MikMod_Read;
	        Reader->Core.Get = &MikMod_Get;
	        Reader->Core.Seek = &MikMod_Seek;
	        Reader->Core.Tell = &MikMod_Tell;
   	}


     	module = Player_LoadGeneric((MREADER *)Reader, 128, 0);
     	module->wrap = true;    // The module will restart when it's finished
     	Player_Start(module);


	AudioPortParam params;
	AudioPortConfig config;
	u32 portNum;

	ioPadInit(7);
	ioPadGetData(0, &paddata);

	audioInit();
	params.numChannels = AUDIO_PORT_2CH;
	params.numBlocks = AUDIO_BLOCK_8;
	params.attr = 0;
	params.level = 0.5f;

	audioPortOpen(&params, &portNum);
	audioGetPortConfig(portNum, &config);
	audioPortStart(portNum);

	while(paddata.BTN_CROSS==0)
	{
		ioPadGetData(0, &paddata);

		playOneBlock((u64*)(u64)config.readIndex,(float*)(u64)config.audioDataStart);
	}

	audioPortStop(portNum);
	audioPortClose(portNum);
	audioQuit();

	return 0;
}
